@@ -1,28 +1,3 @@
-# Copyright (c) 2022, Adrian Dusa
-# All rights reserved.
-# 
-# Redistribution and use in source and binary forms, with or without
-# modification, in whole or in part, are permitted provided that the
-# following conditions are met:
-#     * Redistributions of source code must retain the above copyright
-#       notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#     * The names of its contributors may NOT be used to endorse or promote products
-#       derived from this software without specific prior written permission.
-# 
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-# DISCLAIMED. IN NO EVENT SHALL ADRIAN DUSA BE LIABLE FOR ANY
-# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
 #' @name exportDDI
 #' @title
 #' Export to a DDI metadata file
@@ -31,78 +6,76 @@
 #' Create a DDI Codebook version 2.5, XML file structure.
 #'
 #' @details
-#' #' The information object is essentially a list having two main list components:
+#' #' The information object can either be a data file (includign an R data
+#' frame) or a list having two main list components:
 #'
-#' - \bold{\code{fileDscr}}, if the data is provided in a subcomponent named
-#' \bold{\code{datafile}}
+#' - **`fileDscr`**, if the data is provided in a subcomponent named
+#' **`datafile`**
 #'
-#' - \bold{\code{dataDscr}}, having as many components as the number of variables
-#' in the (meta)data. For each variable, there should a mandatory subcomponent
-#' called \bold{\code{label}} (that contains the variable's label) and, if the
-#' variable is of a categorical type, another subcomponent called
-#' \bold{\code{values}}.
+#' - **`dataDscr`**, having as many components as the number of variables in the
+#' (meta)data. For each variable, there should a mandatory subcomponent called
+#' **`label`** (that contains the variable's label) and, if the variable is of a
+#' categorical type, another subcomponent called **`labels`**.
 #'
 #' Additional informations about the variables can be specified as further
-#' subcomponents, combining DDI specific data but also other information that might
-#' not be covered by DDI:
+#' subcomponents, combining DDI specific data but also other information that
+#' might not be covered by DDI:
 #'
-#' - \bold{\code{measurement}} is the equivalent of the specific DDI attribute
-#' \bold{\code{nature}} of the \bold{\code{var}} element, and it accepts these
-#' values: \code{"nominal"}, \code{"ordinal"}, \code{"interval"}, \code{"ratio"},
-#' \code{"percent"}, and \code{"other"}.
+#' - **`measurement`** is the equivalent of the specific DDI attribute
+#' **`nature`** of the **`var`** element, which accepts these values:
+#' `"nominal"`, `"ordinal"`, `"interval"`, `"ratio"`, `"percent"`, and
+#' `"other"`.
 #'
-#' - \bold{\code{type}} is useful for multiple reasons. A first one, if the
-#' variable is numerical, is to differentiate between \code{discrete} and
-#' \code{contin} values of the attribute \bold{\code{intrvl}} from the same DDI
-#' element \bold{\code{var}}. Another reason is to help identifying pure string
-#' variables (containing text), when the subcomponent \bold{\code{type}} is equal
-#' to \code{"char"}. It is also used for the subelement \bold{\code{varFormat}} of
-#' the element \bold{\code{var}}. Finally, another reason is to differentiate
-#' between pure categorical (\code{"cat"}) and pure numerical (\code{"num"})
-#' variables, as well as mixed ones, among which \code{"numcat"} referring to a
-#' numerical variable with very few values (such as the number of children), for
-#' which it is possible to also produce a table of frequencies along the numerical
-#' summaries. There are also categorical variables that can be interpreted as
-#' numeric (\code{"catnum"}), such as a Likert type response scale with 7 values,
-#' where numerical summaries are also routinely performed along with the usual
-#' table of frequencies.
+#' - **`type`** is useful for multiple reasons. A first one, if the variable is
+#' numerical, is to differentiate between `discrete` and `continuous` values of
+#' the attribute **`intrvl`** from the same DDI element **`var`**. Another
+#' reason is to help identifying pure string variables (containing text), when
+#' the subcomponent **`type`** is equal to `"char"`. It is also used for the
+#' subelement **`varFormat`** of the element **`var`**. Finally, another reason
+#' is to differentiate between pure categorical (`"cat"`) and pure numerical
+#' (`"num"`) variables, as well as mixed ones, among which `"numcat"` referring
+#' to a numerical variable with very few values (such as the number of
+#' children), for which it is possible to also produce a table of frequencies
+#' along the numerical summaries. There are also categorical variables that can
+#' be interpreted as numeric (`"catnum"`), such as a Likert type response scale
+#' with 7 values, where numerical summaries are also routinely performed along
+#' with the usual table of frequencies.
 #'
-#' - \bold{\code{missing}} is an important subcomponent, indicating which of the
-#' values in the variable are going to be treated as missing values, and it is
-#' going to be exported as the attribute \code{missing} of the DDI subelement
-#' \bold{\code{catgry}}.
+#' - **`missing`** is an important subcomponent, indicating which of the values
+#' in the variable are going to be treated as missing values, and it is going to
+#' be exported as the attribute `missing` of the DDI subelement **`catgry`**.
 #'
 #' There are many more possible attributes and DDI elements to be added in the
 #' information object, future versions of this function will likely expand.
 #'
-#' For the moment, only DDI codebook version 2.5 is exported, and DDI Lifecycle is
-#' planned for future releases.
+#' For the moment, only DDI codebook version 2.5 is exported, and DDI Lifecycle
+#' is planned for future releases.
 #'
-#' Argument \bold{\code{xmlang}} expects a two letter ISO country coding, for
-#' instance \code{"en"} to indicate English, or \code{"ro"} to indicate Romanian etc.
+#' Argument **`xmlang`** expects a two letter ISO country coding, for instance
+#' `"en"` to indicate English, or `"ro"` to indicate Romanian etc.
 #'
 #' If the document is monolang, this argument is placed a single time for the
-#' entire document, in the attributes of the \code{codeBook} element. For
+#' entire document, in the attributes of the `codeBook` element. For
 #' multilingual documents, it is placed in the attributes of various other
-#' (sub)elements, for instance \code{abstract} as an obvious one, or the study
+#' (sub)elements, for instance `abstract` as an obvious one, or the study
 #' title, name of the distributing institution, variable labels etc.
 #'
-#' The argument \bold{\code{OS}} can be either:\cr
-#' \code{"windows"} (default), or \code{"Windows"}, \code{"Win"}, \code{"win"},\cr
-#' \code{"MacOS"}, \code{"Darwin"}, \code{"Apple"}, \code{"Mac"}, \code{"mac"},\cr
-#' \code{"Linux"}, \code{"linux"}.
+#' The argument **`OS`** can be either:\cr
+#' `"windows"` (default), or `"Windows"`, `"Win"`, `"win"`,\cr
+#' `"MacOS"`, `"Darwin"`, `"Apple"`, `"Mac"`, `"mac"`,\cr
+#' `"Linux"`, `"linux"`.
 #'
-#' The end of line separator changes only when the target OS is different from the
-#' running OS.
+#' The end of line separator changes only when the target OS is different from
+#' the running OS.
 #'
-#' The argument \bold{\code{indent}} controls how many spaces will be used in the
-#' XML file, to indent the different subelements.
+#' The argument **`indent`** controls how many spaces will be used in the XML
+#' file, to indent the different subelements.
 #'
 #' A small number of required DDI specific elements and attributes have generic
-#' default values but they may be specified using the three dots \bold{\code{...}}
-#' argument. For the current version, these are: \code{IDNo}, \code{titl},
-#' \code{agency}, \code{URI} (for the \code{holdings} element), \code{distrbtr},
-#' \code{abstract} and \code{level} (for the \code{otherMat} element).
+#' default values but they may be specified using the three dots **`...`**
+#' argument. For the current version, these are: `IDNo`, `titl`, `agency`, `URI`
+#' (for the `holdings` element), `distrbtr`, `abstract` and `level` (for the
+#' `otherMat` element).
 #'
 #' @return
 #' An XML file containing a DDI version 2.5 metadata.
@@ -163,11 +136,11 @@
 #' V5 = list(
 #'     label = "Political party reference",
 #'     type = "char",
-#'     txt = "When the respondent indicated his political party reference, his/her
-#'         open response was recoded on a scale of 1-99 with parties with a
-#'         left-wing orientation coded on the low end of the scale and parties with
-#'         a right-wing orientation coded on the high end of the scale. Categories
-#'         90-99 were reserved miscellaneous responses."
+#'     txt = "When the respondent indicated his political party reference,
+#'         his/her open response was recoded on a scale of 1-99 with parties
+#'         with a left-wing orientation coded on the low end of the scale and
+#'         parties with a right-wing orientation coded on the high end of the
+#'         scale. Categories 90-99 were reserved miscellaneous responses."
 #' )))
 #'
 #' \dontrun{
@@ -269,9 +242,19 @@
 
     `replaceChars` <- function(x) {
         x <- replaceTicks(x)
-        x <- gsub("&", "&amp;", x)
-        x <- gsub("<", "&lt;", x)
-        x <- gsub(">", "&gt;", x)
+        x <- gsub(
+            rawToChar(as.raw(c(194, 160))), " ",
+            gsub(
+                "<", "&lt;",
+                gsub(
+                    ">", "&gt;",
+                    gsub(
+                        "&", "&amp;",
+                        x
+                    )
+                )
+            )
+        )
         return(x)
     }
 
@@ -296,6 +279,7 @@
     if (is.null(ns)) ns <- xmlns
     check_arg("xmlns", ns, "c")
 
+
     s0 <- repeatSpace(0)
     s1 <- repeatSpace(1)
     s2 <- repeatSpace(2)
@@ -303,17 +287,17 @@
     s4 <- repeatSpace(4)
     s5 <- repeatSpace(5)
 
-    `catText` <- function(x, ...) {
-        cat(paste(
-            repeatSpace(x),
-            paste(
-                unlist(list(...)),
-                collapse = ""
-            ),
-            enter,
-            sep = ""
-        ))
-    }
+    # `catText` <- function(x, ...) {
+    #     cat(paste(
+    #         repeatSpace(x),
+    #         paste(
+    #             unlist(list(...)),
+    #             collapse = ""
+    #         ),
+    #         enter,
+    #         sep = ""
+    #     ))
+    # }
 
     if (OS == "") {
         OS <- Sys.info()[["sysname"]]
@@ -329,7 +313,7 @@
     # uuid for all variables
     uuid <- generateUUID(length(dataDscr))
 
-    prodate <- as.character(Sys.time())
+    prodDate <- as.character(Sys.time())
     version <- as.character(packageVersion("DDIwR"))
     varnames <- names(dataDscr)
 
@@ -338,9 +322,6 @@
         on.exit(sink())
     }
 
-    prodDate <- as.character(Sys.time())
-    version <- as.character(packageVersion("DDIwR"))
-    varnames <- names(dataDscr)
     cat(paste(
         s0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>",
         enter,
@@ -403,6 +384,7 @@
     cat(paste(s2, "</", ns, "citation>", enter, sep = ""))
     cat(paste(s1, "</", ns, "docDscr>", enter, sep = ""))
 
+
     # Study description
     if (is.null(stdyDscr)) {
         cat(paste(s1, "<", ns, "stdyDscr>", enter, sep = ""))
@@ -438,7 +420,8 @@
         cat(paste(s2, "</", ns, "citation>", enter, sep = ""))
         cat(paste(s2, "<", ns, "stdyInfo>", enter, sep = ""))
         cat(paste(
-            s3, "<", ns, "abstract", xmlang, ">", abstract, "</", ns, "abstract>",
+            s3,
+            "<", ns, "abstract", xmlang, ">", abstract, "</", ns, "abstract>",
             enter, sep = ""
         ))
 
@@ -486,46 +469,76 @@
         }
         else if (!identical(toupper(names(data)), toupper(names(dataDscr)))) {
             admisc::stopError(
-                "Variables in the data do not match the variables in the data description."
+                paste(
+                    "Variables in the data do not match",
+                    "the variables in the data description."
+                )
             )
         }
 
         cat(paste(s2, "<", ns, "fileTxt>", enter, sep = ""))
         if (!is.null(fileName <- codebook[["fileDscr"]][["fileName"]])) {
-            cat(paste(s3, "<", ns, "fileName>", fileName, "</", ns, "fileName>", enter, sep = ""))
+            cat(paste(
+                s3,
+                "<", ns, "fileName>", fileName, "</", ns, "fileName>",
+                enter,
+                sep = ""
+            ))
         }
         cat(paste(s3, "<", ns, "dimensns>", enter, sep = ""))
-        cat(paste(s4, "<", ns, "caseQnty>", nrow(data), "</", ns, "caseQnty>", enter, sep = ""))
-        cat(paste(s4, "<", ns, "varQnty>", ncol(data), "</", ns, "varQnty>", enter, sep = ""))
+        cat(paste(
+            s4,
+            "<", ns, "caseQnty>", nrow(data), "</", ns, "caseQnty>",
+            enter,
+            sep = ""
+        ))
+        cat(paste(
+            s4,
+            "<", ns, "varQnty>", ncol(data), "</", ns, "varQnty>",
+            enter,
+            sep = ""
+        ))
         cat(paste(s3, "</", ns, "dimensns>", enter, sep = ""))
         if (!is.null(fileType <- codebook[["fileDscr"]][["fileType"]])) {
-            cat(paste(s3, "<", ns, "fileType>", fileType, "</", ns, "fileType>", enter, sep = ""))
+            cat(paste(
+                s3,
+                "<", ns, "fileType>", fileType, "</", ns, "fileType>",
+                enter,
+                sep = ""
+            ))
         }
         cat(paste(s2, "</", ns, "fileTxt>", enter, sep = ""))
 
         if (embed) {
-            cat(paste(s2, "<", ns, "notes>", enter, sep = ""))
+            cat(paste(s2, "<", ns, "notes level=\"file\" subject=\"CSV dataset\">", enter, sep = ""))
             cat(paste(
-                s0, "<![CDATA[# start data #",
+                s3, "<![CDATA[# start data #",
                 enter,
                 sep = ""
             ))
 
-            sink()
+            tt <- tempfile()
+            # sink()
             suppressWarnings(
                 write.table(
                     undeclare(data, drop = TRUE),
-                    file = file,
+                    file = tt,
                     sep = ",",
                     na = "",
-                    row.names = FALSE,
-                    append = TRUE
+                    # append = TRUE,
+                    row.names = FALSE
                 )
             )
-            sink(file, append = TRUE)
+            # sink(file, append = TRUE)
 
-            cat(paste(
-                "# end data #", enter, "]]>",
+            cat(
+                paste(s3, readLines(tt), collapse = enter, sep = ""),
+                enter,
+                sep = ""
+            )
+
+            cat(paste(s3,
+                "# end data #]]>",
                 enter,
                 sep = ""
             ))
@@ -555,21 +568,24 @@
 
     for (i in seq(length(dataDscr))) {
         dcml <- ""
-        if (!is.null(data)) {
+        if (!is.null(data) && pN[i]) {
             dcml <- paste0(
                 " dcml=\"",
-                ifelse(
-                    pN[[names(dataDscr)[i]]],
-                    getDecimals(na.omit(aN[[names(dataDscr)[i]]])),
-                    0
-                ),
+                admisc::numdec(na.omit(aN[[varnames[i]]])),
                 "\""
             )
         }
-
+    
         nature <- ""
-        if(any(grepl("measurement", names(dataDscr[[i]])))) {
-            nature <- paste0(" nature=\"", dataDscr[[i]]$measurement, "\"")
+        if (is.element("measurement", names(dataDscr[[i]]))) {
+            nature <- paste0(
+                " nature=\"",
+                gsub(
+                    "categorical|quantitative|, ", "",
+                    dataDscr[[i]]$measurement,
+                ),
+                "\""
+            )
         }
 
         cat(paste0(
@@ -594,6 +610,7 @@
             }
         }
 
+
         na_values <- NULL
         if (is.element("na_values", names(dataDscr[[i]]))) {
             na_values <- dataDscr[[i]]$na_values
@@ -604,6 +621,7 @@
             na_range <- dataDscr[[i]]$na_range
         }
 
+
         if (length(na_range) > 0) {
             cat(paste(s3, "<", ns, "invalrng>", enter, sep = ""))
 
@@ -647,23 +665,15 @@
         }
 
         lbls <- dataDscr[[i]][["labels"]]
-        if (!is.null(lbls)) {
-            nms <- names(lbls)
-
-            # TODO: XML doesn't seem to cope well with multibyte characters?
-            # TODO: what about multibyte languages (e.g. Asian)
-            multibyte <- grepl("[^!-~ ]", lbls)
-            if (any(multibyte)) {
-                for (m in which(multibyte)) {
-                    strlab <- unlist(strsplit(lbls[m], split = ""))
-                    strlab <- strlab[!grepl("[^!-~ ]", strlab)]
-                    lbls[m] <- paste(strlab, collapse = "")
-                }
-            }
-
-            lbls <- setNames(admisc::trimstr(lbls), nms)
-            # names(lbls) <- nms
-        }
+        # if (!is.null(lbls)) {
+        #     # non-ASCII also means non multibyte characters
+        #     lbls <- setNames(
+        #         admisc::trimstr(
+        #             iconv(lbls, "UTF-8", "ASCII", sub = "")
+        #         ),
+        #         names(lbls)
+        #     )
+        # }
 
         type <- dataDscr[[i]]$type
 
@@ -755,20 +765,25 @@
             }
         }
 
+
         if (!is.null(lbls)) {
 
-            tbl <- table(data[[names(dataDscr)[i]]]) # what is the difference from data[[i]] ?
+            # what is the difference from data[[i]] ?
+            tbl <- table(data[[names(dataDscr)[i]]])
 
             for (v in seq(length(lbls))) {
 
                 ismiss <- is.element(lbls[v], na_values)
-                if (length(na_range) > 0) {
-                    ismiss <- ismiss | (lbls[v] >= na_range[1] & lbls[v] <= na_range[2])
+                if (length(na_range) > 0 & pN[i]) {
+                    ismiss <- ismiss | (
+                        lbls[v] >= na_range[1] & lbls[v] <= na_range[2]
+                    )
                 }
 
                 cat(paste(
                     s3,
-                    "<", ns, "catgry", ifelse(ismiss, " missing=\"Y\"", ""), ">",
+                    "<", ns, "catgry",
+                    ifelse(ismiss, " missing=\"Y\"", ""), ">",
                     enter,
                     sep = ""
                 ))
@@ -776,7 +791,7 @@
                 cat(paste(
                     s4,
                     "<", ns, "catValu>",
-                    lbls[v],
+                    replaceChars(lbls[v]),
                     "</", ns, "catValu>",
                     enter,
                     sep = ""
@@ -846,7 +861,12 @@
     }
 
     cat(paste(s1, "</", ns, "dataDscr>", enter, sep = ""))
-    cat(paste(s1, "<", ns, "otherMat level=\"", level, "\"></", ns, "otherMat>", enter, sep = ""))
+    cat(paste(
+        s1,
+        "<", ns, "otherMat level=\"", level, "\"></", ns, "otherMat>",
+        enter,
+        sep = ""
+    ))
     cat(paste(s0, "</", ns, "codeBook>", enter, sep = ""))
 
 }
